@@ -19,6 +19,7 @@
 
 #include "seed-private.h"
 #include <dlfcn.h>
+#include <pthread.h>
 
 JSClassRef gobject_class;
 JSClassRef gobject_method_class;
@@ -41,6 +42,9 @@ seed_toggle_ref (gpointer data, GObject * object, gboolean is_last_ref)
 {
   JSValueRef wrapper;
 
+  if (!g_object_get_data (object, "js-ref"))
+    return;
+
   wrapper = (JSValueRef) data;
 
   if (is_last_ref)
@@ -96,8 +100,8 @@ seed_wrap_object (JSContextRef ctx, GObject * object)
   if (user_data)
     return user_data;
 
-  if (seed_next_gobject_wrapper)
-    js_ref = seed_next_gobject_wrapper;
+  if (pthread_getspecific(seed_next_gobject_wrapper_key))
+    js_ref = pthread_getspecific(seed_next_gobject_wrapper_key);
   else
     js_ref = seed_make_wrapper_for_type (ctx, type);
 
@@ -111,7 +115,7 @@ seed_wrap_object (JSContextRef ctx, GObject * object)
 
   seed_add_signals_to_object (ctx, js_ref, object);
 
-  seed_next_gobject_wrapper = NULL;
+  pthread_setspecific(seed_next_gobject_wrapper_key, NULL);
 
   return js_ref;
 }
@@ -1048,7 +1052,7 @@ seed_gvalue_from_seed_value (JSContextRef ctx,
 
       if (o == NULL || g_type_is_a (G_OBJECT_TYPE (o), type))
 	{
-	  g_value_init (ret, G_TYPE_OBJECT);
+	  g_value_init (ret, type);
 	  g_value_set_object (ret, o);
 
 	  return TRUE;
